/*
    Fichier base64.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions permettant d'encoder et de
    dcoder le contenu d'un tableau en base64.
*/

#include "base64.h"


#define  octet    unsigned char



/* encode base64 nbcar caractres mmoriss
   dans orig et met le rsultat dans dest */

void encode64 (char *orig, char *dest, int nbcar)
{
    octet triplet [3]; // groupe de 3 caractres  convertir en base 64
    int   cartriplet;  // nombre de caractres du triplet rcuprs
    octet valcode [4]; // valeur des 4 caractres en base 64
    int   posorig;     // position dans la ligne passe en paramtre
    int   posdest;     // position dans la ligne contenant le rsultat
    char  valcar [] =  // tableau d'encodage
          "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";


    // initialisations
    posorig = 0;
    posdest = 0;

    // tant que non fin de ligne
    while (posorig < nbcar)
    {
        // initialisation
        cartriplet = 0;

        // rcuprer un groupe de 3 caractres
        do
        {
            triplet [cartriplet++] = orig [posorig++];
        }
        while (cartriplet < 3 && posorig < nbcar);

        // encodage des 4 caractres du rsultat

        // prise en compte du premier caractre du triplet
        valcode [0] = triplet [0] >> 2;
        valcode [1] = (triplet [0] & 3) << 4;

        // si 2me caractre du triplet prsent
        if (cartriplet > 1)
        {
            // le prendre en compte
            valcode [1] = valcode [1] | (triplet [1] >> 4);
            valcode [2] = (triplet [1] & 0x0F) << 2;
        }
        else
            // sinon, on mettra un caractre de padding  la place
            valcode [2] = 64;

        // traitement similaire pour le 3me caractre du triplet
        if (cartriplet == 3)
        {
            valcode [2] = valcode [2] | (triplet [2] >> 6);
            valcode [3] = triplet [2] & 0x3F;
        }
        else
            valcode [3] = 64;

        // affichage des donnes encodes
        dest [posdest++] = valcar [valcode [0]];
        dest [posdest++] = valcar [valcode [1]];
        dest [posdest++] = valcar [valcode [2]];
        dest [posdest++] = valcar [valcode [3]];
    }

    // fin de l'encodage
    dest [posdest] = '\0';
}



/* dcode le contenu de buffer encod base64, met le rsultat
   dans buffer et retourne le nombre de caractres convertis */

int decode64 (char *buffer)
{
    int  car;      // caractre du fichier
    char valcar [4];   // valeur aprs conversion des caractres
    int  i;            // compteur
    int  posorig;  // position dans la ligne passe en paramtre
    int  posdest;  // position dans la nouvelle ligne gnre


    // initialisations
    posorig = 0;
    posdest = 0;

    // tant que non fin de ligne
    while (buffer [posorig] > ' ' && buffer [posorig] != '=')
    {
        // dcoder la valeur de 4 caractres
        for (i = 0; i < 4 && buffer [posorig] != '='; i++)
        {
            // rcuprer un caractre dans la ligne
            car = buffer [posorig++];

            // dcoder ce caractre
            if ('A' <= car && car <= 'Z')
                valcar [i] = car - 'A';
            else if ('a' <= car && car <= 'z')
                valcar [i] = car + 26 - 'a';
            else if ('0' <= car && car <= '9')
                valcar [i] = car + 52 - '0';
            else if (car == '+')
                valcar [i] = 62;
            else if (car == '/')
                valcar [i] = 63;
        }

        // recopier les caractres correspondants dans le buffer
        buffer [posdest++] = (valcar [0] << 2) | (valcar [1] >> 4);

        // sauf si indicateur de fin de message
        if (i > 2)
        {
            buffer [posdest++] = (valcar [1] << 4) | (valcar [2] >> 2);

            if (i > 3)
                buffer [posdest++] = (valcar [2] << 6) | (valcar [3]);
        }
    }

    // terminer le buffer
    buffer [posdest] = '\0';

    // et retourner le nombre de caractres obtenus
    return (posdest);
}
